#include "simpletools.h"
#include "propeller.h"

void init_RAM();
void open_write();
void close_write();
void open_read();
void close_read();
void write_byte(int32_t byte_out, int32_t address);
int32_t read_byte(int32_t address);

int main() {                  // Entry point
  init_RAM();
  open_write();
  int32_t j, start, end, reg;
  
  j = 0;
  start = CNT;
  while(j < 32768) {
    write_byte(j, j);
    j++;
  }
  end = CNT;
  print("%d writes per second\n", 80000000 / ((end-start) / 32768));    
  close_write();
  
  open_read();
  j = 0;
  while(j < 32768) {
    print("%d: %d\n", j, read_byte(j));
    j++;
  }
  
  j = 0;
  start = CNT;
  while(j < 32768) {
    reg = read_byte(j);
    j++;
  }
  end = CNT;
  close_read(); 
   
  print("%d reads per second\n", 80000000 / ((end-start) / 32768)); 
  
}

void init_RAM() {
  DIRA |= (262143 << 8);      // Set address and control bits to outgoing
  OUTA &= ~67108863;          // Clear address, control, and bus bits
  OUTA |= (1 << 25);          // Raise CE to disable
  OUTA |= (1 << 8);           // Raise WE to prevent write
  OUTA |= (1 << 9);           // Raise OE to prevent read
}

void open_write() {
  DIRA |= 255;                // Set bus to outgoing
  OUTA &= ~(1 << 25);         // Lower CE to enable
}

void close_write() {
  OUTA |= (1 << 25);          // Raise CE to disable
  OUTA &= ~255;               // Clear bus
}

void write_byte(int32_t byte_out, int32_t address) {
  OUTA = (OUTA & 0xfe0003ff) | ((address & 0x7fff) << 10);  // Input address
  OUTA = (OUTA & 0xffffff00) | ((byte_out & 0xff) << 0);    // Write data Byte to bus
  OUTA &= ~(1 << 8);          // Lower WE to begin write sequence
  OUTA |= (1 << 8);           // Raise WE to end write sequence
}

void open_read() {
  DIRA &= ~255;               // Set bus to incoming
  OUTA &= (~(1 << 25));       // Lower CE to enable
}

void close_read() {
  OUTA |= (1 << 25);          // Raise CE to disable
  OUTA &= ~255;               // Clear bus
}

int32_t read_byte(int32_t address) {
  int32_t result = 0;          
  OUTA = (OUTA & 0xfe0003ff) | ((address & 0x7fff) << 10);  // Input address
  OUTA &= (~(1 << 9));        // Lower OE to begin read sequence
  result = INA & 0xff;        // Record data Byte from bus
  OUTA |= (1 << 9);           // Raise OE to end read sequence
  return result;
}